/*
 * file CqAction.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM 
 *
 * com.ibm.rational.wvcm.stp.cq.CqAction
 *
 * (C) Copyright IBM Corporation 2004, 2008.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */
package com.ibm.rational.wvcm.stp.cq;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import java.util.List;
import java.util.Map;

import javax.wvcm.WvcmException;
import javax.wvcm.PropertyNameList.PropertyName;

import com.ibm.rational.wvcm.stp.cq.CqProvider;
import com.ibm.rational.wvcm.stpex.StpExEnumeration;
import com.ibm.rational.wvcm.stpex.StpExEnumerationBase;

/**
 * An object that represents a method to be applied to an {@link CqRecord}.
 * <p>
 * The set of CqActions applicable to a CqRecord can be obtained from its
 * CqRecordType resource. (See
 * {@link CqRecordType#ACTION_LIST CqRecordType.ACTION_LIST})
 * <p>
 * Some Actions require additional arguments when they are applied to an Record.
 * These will be documented in the description of the action types.
 * <p>
 * The user-friendly specification for the location of an action has the form
 * <pre>
 *  <b>cq.action:</b><i>&lt;record-type&gt;</i>/<i>&lt;action-name&gt;</i>@<i>&lt;db-set&gt;</i>/<i>&lt;user-db&gt;</i>
 * </pre>
 */

public interface CqAction 
    extends CqUserDbMember
{
    
    /**
     * The type of an action, which places certain restrictions on its use.
     */
    public static enum Type
        implements StpExEnumeration
    {
        /** When a value for this type is not defined. */
        NIL,

        /** This type of action creates a new record */
        SUBMIT,

        /**
         * This type of action changes an existing record without changing its
         * state
         */
        MODIFY,

        /**
         * This type of action changes the state of a record (and other fields,
         * too)
         */
        CHANGE_STATE,

        /**
         * This type of action marks a record as a duplicate of another.
         * <p>
         * Note. This type of action requires an argument named "original". The
         * value of this argument must be a proxy for the record that has been
         * duplicated.
         */
        DUPLICATE,

        /** This type of action removes the DUPLICATE attribute from a record. */
        UNDUPLICATE,

        /**
         * This type of action creates a new record whose content has been
         * import from another database
         */
        IMPORT,

        /**
         * This type of action physically removes a record from the database, as
         * opposed to putting it into a "deleted" state. Actions of this type
         * may only be used with CqRecord.doUnbindAll. Once this action is
         * started, the targeted record is no longer accessible in its change
         * context unless the change context is reverted.
         */
        DELETE,

        /** This type of action is used for defining base hook programs */
        BASE,

        /**
         * For this type of action, the entire body is specified by a
         * record-script.
         */
        RECORD_SCRIPT_ALIAS,

        /**
         * This type of action copies a record with it's children, preserving 
         * the state of the record.
         */
        COPY,
        ;
    }

    /**
     * Returns the Map containing the argument name/value pairs for by this
     * Action.
     * 
     * @return The argument/value Map defined for this action. Will be <b>null</b>
     *         if no such Map has been established.
     */
    Map<String, Object> argumentMap();

    /**
     * Sets any arguments needed when applying this action to an actionable
     * resource.
     * 
     * @param args A Map associating each argument name with its corresponding
     *            value. May be <b>null</b> or empty if this action does not
     *            require any arguments.
     */
    void argumentMap(Map<String, Object> args);

    /**
     * The  {@link Type  Type}  of this action. 
     */
    PropertyName<Type> TYPE = new PropertyName<Type>(PROPERTY_NAMESPACE, "type");

    /**
     * Returns the value of the {@link #TYPE TYPE} property as defined by this
     * proxy.
     *
     * @return  A Type enumerator identifying the type of this action.
     *
     * @throws  WvcmException  if this proxy does not define a value for the
     *                         {@link #TYPE TYPE} property.
     */
    public Type getType() throws WvcmException;

    /**
     * The name of the state that a record will be in upon successful
     * application of this action to the record.
     */
    PropertyName<String> DESTINATION_STATE =
        new PropertyName<String>(PROPERTY_NAMESPACE,
                                 "destination-state");

    /**
     * Returns the value of the {@link #DESTINATION_STATE DESTINATION_STATE}
     * property as defined by this proxy.
     *
     * @return  A String naming the state in which this action leaves a record
     *          when successfully applied to that record; an empty string if
     *          this action is not a CHANGE_STATE action.
     *
     * @throws  WvcmException  if this proxy does not define a value for the
     *                         {@link #DESTINATION_STATE DESTINATION_STATE}
     *                         property.
     */
    public String getDestinationState() throws WvcmException;

    /** The record type with which this action is associated. */
    PropertyName<CqRecordType> RECORD_TYPE =
        new PropertyName<CqRecordType>(PROPERTY_NAMESPACE,
                                       "record-type");

    /**
     * Returns the value of the {@link #RECORD_TYPE RECORD_TYPE} property as
     * defined by this proxy.
     *
     * @return  A CqRecordType proxy for the record type with which this action is
     *          associated.
     *
     * @throws  WvcmException  if this proxy does not define a value for the
     *                         {@link #RECORD_TYPE RECORD_TYPE} property.
     */
    public CqRecordType getRecordType() throws WvcmException;

    /** A list of the states a record must be in to apply this action to it. */
    PropertyName<List<String>> SOURCE_STATE_LIST =
        new PropertyName<List<String>>(PROPERTY_NAMESPACE, "source-state-list");

    /**
     * Returns the value of the {@link #SOURCE_STATE_LIST SOURCE_STATE_LIST}
     * property as defined by this proxy.
     *
     * @return  A list of Strings, each containing the name of a state in which
     *          this action may be applied; an empty list if the action is not
     *          a CHANGE_STATE action.
     *
     * @throws  WvcmException  if this proxy does not define a value for the
     *                         {@link #SOURCE_STATE_LIST SOURCE_STATE_LIST}
     *                         property.
     */
    public List<String> getSourceStateList() throws WvcmException;
}
